import pytest
from mm_pos.menu import MenuItem
from mm_pos.order import Order


def test_add_item_and_total():
    order = Order(table_number=5)
    burger = MenuItem("Burger", 8.99, "Food")
    fries = MenuItem("Fries", 3.49, "Food")

    order.add_item(burger, qty=2)  # 2 burgers
    order.add_item(fries, qty=1)  # 1 fries

    assert order.total() == pytest.approx(8.99 * 2 + 3.49)


def test_order_summary_format():
    order = Order()
    cola = MenuItem("Cola", 1.99, "Drink")
    order.add_item(cola, qty=3)

    summary = order.summary()
    assert "3x Cola - $5.97" in summary
    assert "Total: $5.97" in summary


def test_empty_order_total_is_zero():
    order = Order()
    assert order.total() == 0
    assert "Total: $0.00" in order.summary()



from mm_pos.menu import MenuItem
from mm_pos.order import Order
from mm_pos.receipt import Receipt
from mm_pos.payment import Payment


def test_receipt_contains_items_and_total():
    order = Order(table_number=3)
    burger = MenuItem("Burger", 8.99, "Food")
    cola = MenuItem("Cola", 1.99, "Drink")

    order.add_item(burger, qty=2)
    order.add_item(cola, qty=1)

    receipt = Receipt(order).generate()

    # Check important details are in the receipt
    assert "Table: 3" in receipt
    assert "2x Burger - $17.98" in receipt
    assert "1x Cola - $1.99" in receipt
    assert "Total: $19.97" in receipt
    assert "Thank you for dining with us!" in receipt


def test_receipt_includes_payment_info():
    order = Order(table_number=2)
    burger = MenuItem("Burger", 10.00)
    order.add_item(burger, qty=1)

    payment = Payment(order, method="cash", amount_given=20.00)
    receipt = Receipt(order, payment=payment).generate()

    assert "Total: $10.00" in receipt
    assert "Paid Cash - Change Due: $10.00" in receipt



from mm_pos.db import init_db, UserDB, OrderDB, PaymentDB, MenuItemDB, OrderItemDB


def test_user_with_multiple_orders_and_payments(tmp_path):
    db_path = tmp_path / "user_test.db"
    Session = init_db(f"sqlite:///{db_path}")
    session = Session()

    # Create a user (waiter)
    alice = UserDB(name="Alice", role="waiter")
    alice.set_pin("1234")  # ✅ hash the pin before saving
    session.add(alice)
    session.commit()

    # Create menu item
    burger = MenuItemDB(name="Burger", price=10.00, category="Food")
    session.add(burger)
    session.commit()

    # First order by Alice
    order1 = OrderDB(table_number=1, user=alice)
    session.add(order1)
    session.commit()

    session.add(OrderItemDB(order_id=order1.id, menu_item_id=burger.id, qty=2))
    session.commit()

    payment1 = PaymentDB(order=order1, method="cash", amount_given=30.0, user=alice)
    session.add(payment1)
    session.commit()

    # Second order by Alice
    order2 = OrderDB(table_number=2, user=alice)
    session.add(order2)
    session.commit()

    session.add(OrderItemDB(order_id=order2.id, menu_item_id=burger.id, qty=1))
    session.commit()

    payment2 = PaymentDB(order=order2, method="card", user=alice)
    session.add(payment2)
    session.commit()

    # Assertions
    saved_user = session.query(UserDB).first()
    assert saved_user.name == "Alice"
    assert len(saved_user.orders) == 2
    assert len(saved_user.payments) == 2

    # Check roles and linkage
    assert saved_user.orders[0].user == saved_user
    assert saved_user.payments[0].user == saved_user


def test_user_role_permissions(tmp_path):
    db_path = tmp_path / "roles_test.db"
    Session = init_db(f"sqlite:///{db_path}")
    session = Session()

    waiter = UserDB(name="Wally", role="waiter")
    waiter.set_pin("1111")  # ✅ must set pin
    cashier = UserDB(name="Cathy", role="cashier")
    cashier.set_pin("2222")
    admin = UserDB(name="Alice", role="admin")
    admin.set_pin("3333")

    session.add_all([waiter, cashier, admin])
    session.commit()

    # Waiter
    assert waiter.can_take_orders() is True
    assert waiter.can_process_payments() is False
    assert waiter.can_view_reports() is False

    # Cashier
    assert cashier.can_take_orders() is True
    assert cashier.can_process_payments() is True
    assert cashier.can_view_reports() is False

    # Admin
    assert admin.is_admin() is True
    assert admin.can_take_orders() is True
    assert admin.can_process_payments() is True
    assert admin.can_view_reports() is True



from sqlalchemy.orm import Session
from mm_pos.db import TableDB


class TableManager:
    def __init__(self, session: Session):
        self.session = session

    def open_table(self, number: int):
        """Open a new table if not already in use."""
        table = self.session.query(TableDB).filter_by(number=number).first()
        if table:
            if table.status != "closed":
                raise ValueError(f"Table {number} is already {table.status}.")
            table.status = "open"
        else:
            table = TableDB(number=number, status="open")
            self.session.add(table)
        self.session.commit()
        return table

    def occupy_table(self, number: int):
        """Mark a table as occupied."""
        table = self.session.query(TableDB).filter_by(number=number).first()
        if not table:
            raise ValueError(f"Table {number} does not exist.")
        table.status = "occupied"
        self.session.commit()
        return table

    def close_table(self, number: int):
        """Mark a table as closed (after bill is settled)."""
        table = self.session.query(TableDB).filter_by(number=number).first()
        if not table:
            raise ValueError(f"Table {number} does not exist.")
        table.status = "closed"
        self.session.commit()
        return table

    def merge_tables(self, table1_num: int, table2_num: int):
        """Merge two tables into one (combine orders)."""
        t1 = self.session.query(TableDB).filter_by(number=table1_num).first()
        t2 = self.session.query(TableDB).filter_by(number=table2_num).first()
        if not t1 or not t2:
            raise ValueError("Both tables must exist to merge.")
        if t1.status == "closed" or t2.status == "closed":
            raise ValueError("Cannot merge closed tables.")

        # Move orders from t2 → t1
        for order in t2.orders:
            order.table = t1
        t2.status = "closed"
        self.session.commit()
        return t1



from datetime import date
from mm_pos.db import init_db, MenuItemDB, OrderDB, OrderItemDB, PaymentDB
from mm_pos.reports import Reports


def test_reports_generate_basic_stats(tmp_path):
    db_path = tmp_path / "reports_test.db"
    Session = init_db(f"sqlite:///{db_path}")
    session = Session()

    # Add menu items
    burger = MenuItemDB(name="Burger", price=10.0)
    cola = MenuItemDB(name="Cola", price=2.0)
    session.add_all([burger, cola])
    session.commit()

    # Create order with 2 burgers + 1 cola
    order1 = OrderDB(table_number=1)
    session.add(order1)
    session.commit()

    session.add(OrderItemDB(order_id=order1.id, menu_item_id=burger.id, qty=2))
    session.add(OrderItemDB(order_id=order1.id, menu_item_id=cola.id, qty=1))
    session.commit()

    # Add payment
    payment = PaymentDB(order_id=order1.id, method="cash", amount_given=25.0)
    session.add(payment)
    session.commit()

    reports = Reports(session)

    # Daily sales should equal 2*10 + 1*2 = 22
    assert reports.daily_sales_total(date.today()) == 22.0

    # Top selling item should be Burger (2 qty)
    top_items = reports.top_selling_items(limit=1)
    assert top_items[0][0] == "Burger"
    assert top_items[0][1] == 2

    # Payment breakdown should show 1 cash payment
    breakdown = reports.payment_breakdown(date.today())
    assert breakdown.get("cash") == 1


def test_reports_csv_exports(tmp_path):
    db_path = tmp_path / "reports_csv.db"
    Session = init_db(f"sqlite:///{db_path}")
    session = Session()

    # Setup menu and order
    burger = MenuItemDB(name="Burger", price=10.0)
    session.add(burger)
    session.commit()

    order = OrderDB(table_number=1)
    session.add(order)
    session.commit()

    session.add(OrderItemDB(order_id=order.id, menu_item_id=burger.id, qty=3))
    session.commit()

    session.add(PaymentDB(order_id=order.id, method="card"))
    session.commit()

    reports = Reports(session)

    # Export top items
    items_csv = tmp_path / "top_items.csv"
    reports.export_top_items_csv(str(items_csv))
    assert items_csv.exists()

    with open(items_csv) as f:
        content = f.read()
        assert "Burger" in content
        assert "3" in content

    # Export payment breakdown
    payments_csv = tmp_path / "payments.csv"
    reports.export_payment_breakdown_csv(str(payments_csv), date.today())
    assert payments_csv.exists()

    with open(payments_csv) as f:
        content = f.read()
        assert "card" in content
        assert "1" in content



import pytest
from fastapi.testclient import TestClient
from mm_pos.api import app, session
from mm_pos.db import Base, get_engine

client = TestClient(app)


@pytest.fixture(autouse=True)
def setup_db(tmp_path):
    """Recreate a fresh DB for every test run."""
    db_path = tmp_path / "test_api.db"
    engine = get_engine(f"sqlite:///{db_path}")
    Base.metadata.drop_all(engine)
    Base.metadata.create_all(engine)
    session.bind = engine
    yield


def register_and_login(name: str, role: str, pin: str):
    """Helper: register user + login to get auth token."""
    r = client.post("/register/", data={"name": name, "role": role, "pin": pin})
    assert r.status_code == 200

    r = client.post("/login/", data={"name": name, "pin": pin})
    assert r.status_code == 200
    token = r.json()["access_token"]
    return {"Authorization": f"Bearer {token}"}


# --- Auth Tests ---


def test_register_and_login():
    headers = register_and_login("Alice", "admin", "1234")
    assert "Authorization" in headers


# --- Menu Tests ---


def test_menu_requires_admin():
    # Waiter should NOT be able to add menu
    waiter_headers = register_and_login("Wally", "waiter", "1111")
    r = client.post(
        "/menu/", params={"name": "Burger", "price": 9.99}, headers=waiter_headers
    )
    assert r.status_code == 403

    # Admin can add
    admin_headers = register_and_login("Alice", "admin", "1234")
    r = client.post(
        "/menu/", params={"name": "Burger", "price": 9.99}, headers=admin_headers
    )
    assert r.status_code == 200
    assert r.json()["name"] == "Burger"

    # Menu list should include Burger
    r = client.get("/menu/", headers=admin_headers)
    assert any(i["name"] == "Burger" for i in r.json())


# --- Orders Tests ---


def test_create_order_and_add_item():
    admin_headers = register_and_login("Alice", "admin", "1234")
    client.post("/menu/", params={"name": "Cola", "price": 2.5}, headers=admin_headers)

    # Cashier can create order
    cashier_headers = register_and_login("Cathy", "cashier", "2222")
    r = client.post("/orders/", params={"table_number": 5}, headers=cashier_headers)
    assert r.status_code == 200
    order_id = r.json()["id"]

    # Add item to order
    r = client.post(
        f"/orders/{order_id}/items/",
        params={"menu_item_id": 1, "qty": 2},
        headers=cashier_headers,
    )
    assert r.status_code == 200
    assert r.json()["qty"] == 2


# --- Payments Tests ---


def test_payment_requires_cashier_or_admin():
    admin_headers = register_and_login("Alice", "admin", "1234")
    client.post("/menu/", params={"name": "Soup", "price": 5.0}, headers=admin_headers)
    r = client.post("/orders/", params={"table_number": 1}, headers=admin_headers)
    order_id = r.json()["id"]

    waiter_headers = register_and_login("Wally", "waiter", "1111")
    r = client.post(
        "/payments/",
        params={"order_id": order_id, "method": "cash"},
        headers=waiter_headers,
    )
    assert r.status_code == 403  # waiter not allowed

    cashier_headers = register_and_login("Cathy", "cashier", "2222")
    r = client.post(
        "/payments/",
        params={"order_id": order_id, "method": "cash"},
        headers=cashier_headers,
    )
    assert r.status_code == 200


# --- Reports Tests ---


def test_reports_only_admin():
    admin_headers = register_and_login("Alice", "admin", "1234")
    r = client.get("/reports/daily/", headers=admin_headers)
    assert r.status_code == 200

    cashier_headers = register_and_login("Cathy", "cashier", "2222")
    r = client.get("/reports/daily/", headers=cashier_headers)
    assert r.status_code == 403


# --- Tables Tests ---


def test_open_and_close_table():
    admin_headers = register_and_login("Alice", "admin", "1234")

    # Open
    r = client.post("/tables/open/", params={"number": 10}, headers=admin_headers)
    assert r.status_code == 200
    assert r.json()["status"] == "open"

    # Close
    r = client.post("/tables/close/", params={"number": 10}, headers=admin_headers)
    assert r.status_code == 200
    assert r.json()["status"] == "closed"



# mm_pos/payment.py

from mm_pos.order import Order


class Payment:
    def __init__(self, order: Order, method: str, amount_given: float = None):
        self.order = order
        self.method = method.lower()
        self.amount_given = amount_given

        if self.method == "cash" and (
            amount_given is None or amount_given < order.total()
        ):
            raise ValueError("Insufficient cash provided for this order.")

    def change_due(self) -> float:
        if self.method != "cash":
            return 0.0
        return round(self.amount_given - self.order.total(), 2)

    def summary(self) -> str:
        if self.method == "cash":
            return f"Paid {self.method.title()} - Change Due: ${self.change_due():.2f}"
        return f"Paid {self.method.title()}"



import pytest
from mm_pos.menu import MenuItem
from mm_pos.order import Order
from mm_pos.payment import Payment


def make_order():
    order = Order(table_number=1)
    order.add_item(MenuItem("Burger", 10.00))
    order.add_item(MenuItem("Cola", 2.00))
    return order


def test_cash_payment_with_change():
    order = make_order()
    payment = Payment(order, method="cash", amount_given=20.00)
    assert payment.change_due() == pytest.approx(8.00)
    assert "Change Due: $8.00" in payment.summary()


def test_cash_payment_insufficient_amount():
    order = make_order()
    with pytest.raises(ValueError):
        Payment(order, method="cash", amount_given=5.00)


def test_card_payment_has_no_change():
    order = make_order()
    payment = Payment(order, method="card")
    assert payment.change_due() == 0.0
    assert "Paid Card" in payment.summary()


def test_mobile_payment_summary():
    order = make_order()
    payment = Payment(order, method="mobile")
    assert "Paid Mobile" in payment.summary()



from mm_pos.menu import Menu, MenuItem


def test_add_item_to_menu():
    menu = Menu()
    burger = MenuItem("Burger", 8.99, "Food")
    menu.add_item(burger)
    assert burger in menu.items


def test_list_items_returns_all():
    menu = Menu()
    menu.add_item(MenuItem("Cola", 1.99, "Drink"))
    menu.add_item(MenuItem("Fries", 3.49, "Food"))
    items = menu.list_items()
    assert len(items) == 2
    assert all(isinstance(item, MenuItem) for item in items)


def test_menu_item_repr():
    item = MenuItem("Burger", 8.99, "Food")
    assert "Burger ($8.99)" in repr(item)



# mm_pos/order.py

from mm_pos.menu import MenuItem


class Order:
    def __init__(self, table_number: int = None, takeout: bool = False):
        self.items = []
        self.table_number = table_number
        self.takeout = takeout

    def add_item(self, item: MenuItem, qty: int = 1):
        self.items.append((item, qty))

    def total(self):
        return sum(item.price * qty for item, qty in self.items)

    def summary(self):
        lines = [
            f"{qty}x {item.name} - ${item.price * qty:.2f}" for item, qty in self.items
        ]
        lines.append(f"Total: ${self.total():.2f}")
        return "\n".join(lines)



from fastapi import FastAPI, HTTPException, Depends, status, Form
from fastapi.security import OAuth2PasswordBearer
from jose import jwt, JWTError
from datetime import datetime, timedelta, UTC

from mm_pos.db import (
    init_db,
    MenuItemDB,
    OrderDB,
    OrderItemDB,
    PaymentDB,
    UserDB,
)
from mm_pos.reports import Reports
from mm_pos.tables import TableManager

Session = init_db()
session = Session()

app = FastAPI(title="mm-pos API", version="0.1.0")

# ========================
# Authentication utilities
# ========================

SECRET_KEY = "super-secret-key"  # TODO: replace with env var in production
ALGORITHM = "HS256"
ACCESS_TOKEN_EXPIRE_MINUTES = 60

oauth2_scheme = OAuth2PasswordBearer(tokenUrl="login")


def create_access_token(data: dict, expires_delta: timedelta = None):
    """Create a signed JWT token for authentication."""
    to_encode = data.copy()
    expire = datetime.now(UTC) + (
        expires_delta or timedelta(minutes=ACCESS_TOKEN_EXPIRE_MINUTES)
    )
    to_encode.update({"exp": expire})
    return jwt.encode(to_encode, SECRET_KEY, algorithm=ALGORITHM)


def get_current_user(token: str = Depends(oauth2_scheme)):
    """Get the logged-in user from a JWT token."""
    try:
        payload = jwt.decode(token, SECRET_KEY, algorithms=[ALGORITHM])
        user_id: int = int(payload.get("sub"))
    except JWTError:
        raise HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED, detail="Invalid token"
        )

    user = session.get(UserDB, user_id)
    if not user:
        raise HTTPException(status_code=404, detail="User not found")
    return user


# ========================
# Auth endpoints
# ========================


@app.post("/register/")
def register_user(name: str = Form(...), role: str = Form(...), pin: str = Form(...)):
    """Register a new user (waiter, cashier, admin)."""
    user = UserDB(name=name, role=role)
    user.set_pin(pin)  # hash + store pin
    session.add(user)
    session.commit()
    return {"id": user.id, "name": user.name, "role": user.role}


@app.post("/login/")
def login(name: str = Form(...), pin: str = Form(...)):
    """Login with name + PIN, returns a JWT token."""
    user = session.query(UserDB).filter_by(name=name).first()
    if not user or not user.verify_pin(pin):
        raise HTTPException(status_code=400, detail="Invalid credentials")
    token = create_access_token({"sub": str(user.id)})
    return {"access_token": token, "token_type": "bearer"}


# ========================
# Menu endpoints
# ========================


@app.post("/menu/")
def add_menu_item(
    name: str,
    price: float,
    category: str = "General",
    current_user: UserDB = Depends(get_current_user),
):
    if not current_user.is_admin():
        raise HTTPException(status_code=403, detail="Admins only")
    item = MenuItemDB(name=name, price=price, category=category)
    session.add(item)
    session.commit()
    return {
        "id": item.id,
        "name": item.name,
        "price": item.price,
        "category": item.category,
    }


@app.get("/menu/")
def list_menu_items(current_user: UserDB = Depends(get_current_user)):
    items = session.query(MenuItemDB).all()
    return [
        {"id": i.id, "name": i.name, "price": i.price, "category": i.category}
        for i in items
    ]


# ========================
# Orders endpoints
# ========================


@app.post("/orders/")
def create_order(
    table_number: int = None,
    takeout: bool = False,
    current_user: UserDB = Depends(get_current_user),
):
    if not current_user.can_take_orders():
        raise HTTPException(status_code=403, detail="Not authorized to create orders")
    order = OrderDB(table_number=table_number, takeout=takeout, user_id=current_user.id)
    session.add(order)
    session.commit()
    return {
        "id": order.id,
        "table_number": order.table_number,
        "takeout": order.takeout,
        "user_id": current_user.id,
    }


@app.post("/orders/{order_id}/items/")
def add_item_to_order(
    order_id: int,
    menu_item_id: int,
    qty: int = 1,
    current_user: UserDB = Depends(get_current_user),
):
    if not current_user.can_take_orders():
        raise HTTPException(status_code=403, detail="Not authorized to add items")

    order = session.get(OrderDB, order_id)

    if not order:
        raise HTTPException(status_code=404, detail="Order not found")

    item = session.get(MenuItemDB, menu_item_id)
    if not item:
        raise HTTPException(status_code=404, detail="Menu item not found")

    oi = OrderItemDB(order_id=order.id, menu_item_id=item.id, qty=qty)
    session.add(oi)
    session.commit()
    return {"order_id": order.id, "item": item.name, "qty": qty}


# ========================
# Payments endpoints
# ========================


@app.post("/payments/")
def add_payment(
    order_id: int,
    method: str,
    amount_given: float = None,
    current_user: UserDB = Depends(get_current_user),
):
    if not current_user.can_process_payments():
        raise HTTPException(
            status_code=403, detail="Not authorized to process payments"
        )

    order = session.get(OrderDB, order_id)
    if not order:
        raise HTTPException(status_code=404, detail="Order not found")

    payment = PaymentDB(
        order_id=order_id,
        method=method,
        amount_given=amount_given,
        user_id=current_user.id,
    )
    session.add(payment)
    session.commit()
    return {
        "payment_id": payment.id,
        "order_id": order_id,
        "method": method,
        "amount_given": amount_given,
        "user_id": current_user.id,
    }


# ========================
# Reports endpoints
# ========================


@app.get("/reports/daily/")
def daily_sales(current_user: UserDB = Depends(get_current_user)):
    if not current_user.can_view_reports():
        raise HTTPException(status_code=403, detail="Admins only")
    reports = Reports(session)
    return {"total_sales": reports.daily_sales_total()}


@app.get("/reports/top-items/")
def top_items(limit: int = 5, current_user: UserDB = Depends(get_current_user)):
    if not current_user.can_view_reports():
        raise HTTPException(status_code=403, detail="Admins only")
    reports = Reports(session)
    return {"top_items": reports.top_selling_items(limit)}


@app.get("/reports/payments/")
def payment_breakdown(current_user: UserDB = Depends(get_current_user)):
    if not current_user.can_view_reports():
        raise HTTPException(status_code=403, detail="Admins only")
    reports = Reports(session)
    return {"payment_breakdown": reports.payment_breakdown()}


# ========================
# Tables endpoints
# ========================


@app.post("/tables/open/")
def open_table(number: int, current_user: UserDB = Depends(get_current_user)):
    if not current_user.can_take_orders():
        raise HTTPException(status_code=403, detail="Not authorized to open tables")
    manager = TableManager(session)
    t = manager.open_table(number)
    return {"id": t.id, "number": t.number, "status": t.status}


@app.post("/tables/close/")
def close_table(number: int, current_user: UserDB = Depends(get_current_user)):
    if not current_user.can_process_payments():
        raise HTTPException(status_code=403, detail="Not authorized to close tables")
    manager = TableManager(session)
    t = manager.close_table(number)
    return {"id": t.id, "number": t.number, "status": t.status}



# mm_pos/receipt.py

from datetime import datetime
from mm_pos.order import Order
from mm_pos.payment import Payment


class Receipt:
    def __init__(self, order: Order, payment: Payment = None):
        self.order = order
        self.payment = payment
        self.timestamp = datetime.now()

    def generate(self) -> str:
        header = ["MM-POS Receipt", "-" * 20]

        if self.order.table_number:
            header.append(f"Table: {self.order.table_number}")
        elif self.order.takeout:
            header.append("Takeout Order")

        header.append(f"Date: {self.timestamp.strftime('%Y-%m-%d %H:%M:%S')}\n")

        # Items
        body = [
            f"{qty}x {item.name} - ${item.price * qty:.2f}"
            for item, qty in self.order.items
        ]

        # Footer
        footer = [
            "-" * 20,
            f"Total: ${self.order.total():.2f}",
            "Thank you for dining with us!",
        ]

        # Add payment info if available
        if self.payment:
            footer.append(self.payment.summary())

        return "\n".join(header + body + footer)



from sqlalchemy.orm import Session
from mm_pos.db import InventoryDB


class InventoryManager:
    def __init__(self, session: Session, enforce: bool = True):
        """
        :param session: SQLAlchemy session
        :param enforce: whether to enforce strict stock checks (default=True)
        """
        self.session = session
        self.enforce = enforce

    def add_stock(self, name: str, qty: float):
        """Add or update stock quantity for an ingredient."""
        item = self.session.query(InventoryDB).filter_by(name=name).first()
        if not item:
            item = InventoryDB(name=name, quantity=qty)
            self.session.add(item)
        else:
            item.quantity += qty
        self.session.commit()
        return item

    def get_stock(self, name: str) -> float:
        """Return current stock quantity for an ingredient."""
        item = self.session.query(InventoryDB).filter_by(name=name).first()
        return item.quantity if item else 0.0

    def deduct_for_order(self, order_items, enforce: bool = None):
        """
        Deduct inventory for a list of OrderItemDB objects.
        Uses global enforcement unless override provided.
        """
        # Decide enforcement mode
        if enforce is None:
            enforce = self.enforce

        # First pass: check stock only if links exist
        for oi in order_items:
            if not oi.menu_item.inventory_links:
                continue  # skip items with no inventory tracking

            for link in oi.menu_item.inventory_links:
                required = link.amount_used * oi.qty
                if enforce and link.inventory.quantity < required:
                    raise ValueError(
                        f"Not enough stock for {link.inventory.name}. "
                        f"Required {required}, available {link.inventory.quantity}"
                    )

        # Second pass: deduct
        for oi in order_items:
            if not oi.menu_item.inventory_links:
                continue
            for link in oi.menu_item.inventory_links:
                required = link.amount_used * oi.qty
                link.inventory.quantity -= required
        self.session.commit()

    def low_stock_alerts(self, threshold: float = 5.0):
        """Return a list of items below the given threshold."""
        return [
            (inv.name, inv.quantity)
            for inv in self.session.query(InventoryDB).all()
            if inv.quantity <= threshold
        ]



# mm_pos/menu.py


class MenuItem:
    def __init__(self, name: str, price: float, category: str = "General"):
        self.name = name
        self.price = price
        self.category = category

    def __repr__(self):
        return f"{self.name} (${self.price:.2f})"


class Menu:
    def __init__(self):
        self.items = []

    def add_item(self, item: MenuItem):
        self.items.append(item)

    def list_items(self):
        return self.items



# mm_pos/db.py
from sqlalchemy import (
    create_engine,
    Column,
    Integer,
    String,
    Float,
    ForeignKey,
    DateTime,
    Boolean,
)
from sqlalchemy.orm import declarative_base, sessionmaker, relationship
from datetime import datetime, timezone
from passlib.hash import bcrypt

Base = declarative_base()


class MenuItemDB(Base):
    __tablename__ = "menu_items"
    id = Column(Integer, primary_key=True, autoincrement=True)
    name = Column(String, nullable=False)
    price = Column(Float, nullable=False)
    category = Column(String, default="General")

    inventory_links = relationship(
        "MenuItemInventoryDB", back_populates="menu_item", cascade="all, delete-orphan"
    )


class OrderDB(Base):
    __tablename__ = "orders"
    id = Column(Integer, primary_key=True, autoincrement=True)
    table_number = Column(Integer, nullable=True)
    takeout = Column(Boolean, default=False)
    timestamp = Column(DateTime, default=lambda: datetime.now(timezone.utc))

    table_id = Column(Integer, ForeignKey("tables.id"), nullable=True)
    table = relationship("TableDB", back_populates="orders")

    # FK to user
    user_id = Column(Integer, ForeignKey("users.id"), nullable=True)
    user = relationship("UserDB", back_populates="orders")

    items = relationship(
        "OrderItemDB", back_populates="order", cascade="all, delete-orphan"
    )
    payments = relationship(
        "PaymentDB", back_populates="order", cascade="all, delete-orphan"
    )


class OrderItemDB(Base):
    __tablename__ = "order_items"
    id = Column(Integer, primary_key=True, autoincrement=True)
    order_id = Column(Integer, ForeignKey("orders.id"), nullable=False)
    menu_item_id = Column(Integer, ForeignKey("menu_items.id"), nullable=False)
    qty = Column(Integer, nullable=False, default=1)

    order = relationship("OrderDB", back_populates="items")
    menu_item = relationship("MenuItemDB")


class PaymentDB(Base):
    __tablename__ = "payments"
    id = Column(Integer, primary_key=True, autoincrement=True)
    order_id = Column(Integer, ForeignKey("orders.id"), nullable=False)
    user_id = Column(Integer, ForeignKey("users.id"), nullable=True)
    method = Column(String, nullable=False)
    amount_given = Column(Float, nullable=True)
    timestamp = Column(DateTime, default=lambda: datetime.now(timezone.utc))

    order = relationship("OrderDB", back_populates="payments")
    user = relationship("UserDB", back_populates="payments")


class TableDB(Base):
    __tablename__ = "tables"
    id = Column(Integer, primary_key=True, autoincrement=True)
    number = Column(Integer, unique=True, nullable=False)
    status = Column(String, default="open")  # open, occupied, closed

    orders = relationship(
        "OrderDB", back_populates="table", cascade="all, delete-orphan"
    )


class InventoryDB(Base):
    __tablename__ = "inventory"
    id = Column(Integer, primary_key=True, autoincrement=True)
    name = Column(String, nullable=False, unique=True)  # e.g. "Burger Patty"
    quantity = Column(
        Float, nullable=False, default=0.0
    )  # in units (e.g. pieces, liters)

    # relationship: which menu items depend on this stock
    menu_links = relationship(
        "MenuItemInventoryDB", back_populates="inventory", cascade="all, delete-orphan"
    )


class MenuItemInventoryDB(Base):
    """
    Link table between MenuItem and Inventory.
    Defines how much of each ingredient is used per menu item.
    """

    __tablename__ = "menu_item_inventory"
    id = Column(Integer, primary_key=True, autoincrement=True)
    menu_item_id = Column(Integer, ForeignKey("menu_items.id"), nullable=False)
    inventory_id = Column(Integer, ForeignKey("inventory.id"), nullable=False)
    amount_used = Column(Float, nullable=False)  # how much stock is consumed per item

    menu_item = relationship("MenuItemDB", back_populates="inventory_links")
    inventory = relationship("InventoryDB", back_populates="menu_links")


class UserDB(Base):
    __tablename__ = "users"
    id = Column(Integer, primary_key=True, autoincrement=True)
    name = Column(String, nullable=False)
    role = Column(String, nullable=False)  # waiter, cashier, admin
    pin_hash = Column(String, nullable=False)  # store hashed PIN instead of raw

    orders = relationship(
        "OrderDB", back_populates="user", cascade="all, delete-orphan"
    )
    payments = relationship(
        "PaymentDB", back_populates="user", cascade="all, delete-orphan"
    )

    # --- Auth utilities ---
    def set_pin(self, pin: str):
        self.pin_hash = bcrypt.hash(pin)

    def verify_pin(self, pin: str) -> bool:
        return bcrypt.verify(pin, self.pin_hash)

    # --- Role helpers ---
    def is_admin(self):
        return self.role.lower() == "admin"

    def can_take_orders(self):
        return self.role.lower() in ("waiter", "cashier", "admin")

    def can_process_payments(self):
        return self.role.lower() in ("cashier", "admin")

    def can_view_reports(self):
        return self.role.lower() == "admin"


# --- Setup Functions ---
def get_engine(db_url="sqlite:///mmpos.db"):
    return create_engine(db_url, echo=False)


def init_db(engine=None):
    if engine is None:
        engine = get_engine()
    elif isinstance(engine, str):
        engine = get_engine(engine)

    Base.metadata.create_all(engine)
    return sessionmaker(bind=engine)



# mm_pos/reports.py

import csv
from sqlalchemy.orm import Session
from sqlalchemy import func
from datetime import date
from mm_pos.db import OrderDB, OrderItemDB, MenuItemDB, PaymentDB


class Reports:
    def __init__(self, session: Session):
        self.session = session

    def daily_sales_total(self, day: date = None) -> float:
        """Return total sales for a given day (default: today)."""
        if day is None:
            day = date.today()

        total = (
            self.session.query(func.sum(MenuItemDB.price * OrderItemDB.qty))
            .join(OrderItemDB, MenuItemDB.id == OrderItemDB.menu_item_id)
            .join(OrderDB, OrderItemDB.order_id == OrderDB.id)
            .filter(func.date(OrderDB.timestamp) == day)
            .scalar()
        )
        return round(total or 0.0, 2)

    def top_selling_items(self, limit: int = 5):
        """Return top N selling items (name, quantity)."""
        results = (
            self.session.query(
                MenuItemDB.name, func.sum(OrderItemDB.qty).label("total_qty")
            )
            .join(OrderItemDB, MenuItemDB.id == OrderItemDB.menu_item_id)
            .group_by(MenuItemDB.name)
            .order_by(func.sum(OrderItemDB.qty).desc())
            .limit(limit)
            .all()
        )
        return [(row[0], row[1]) for row in results]

    def payment_breakdown(self, day: date = None):
        """Return total collected per payment method (cash, card, etc.) for a given day."""
        if day is None:
            day = date.today()

        results = (
            self.session.query(
                PaymentDB.method, func.count(PaymentDB.id).label("count")
            )
            .filter(func.date(PaymentDB.timestamp) == day)
            .group_by(PaymentDB.method)
            .all()
        )
        return {row[0]: row[1] for row in results}

    # --- CSV Export Utilities ---
    def export_top_items_csv(self, filepath: str, limit: int = 5):
        """Export top-selling items to a CSV file."""
        items = self.top_selling_items(limit)
        with open(filepath, mode="w", newline="") as csvfile:
            writer = csv.writer(csvfile)
            writer.writerow(["Item", "Quantity Sold"])
            for name, qty in items:
                writer.writerow([name, qty])

    def export_payment_breakdown_csv(self, filepath: str, day: date = None):
        """Export payment breakdown to CSV file."""
        breakdown = self.payment_breakdown(day)
        with open(filepath, mode="w", newline="") as csvfile:
            writer = csv.writer(csvfile)
            writer.writerow(["Payment Method", "Count"])
            for method, count in breakdown.items():
                writer.writerow([method, count])



from mm_pos.db import init_db, OrderDB
from mm_pos.tables import TableManager


def test_table_lifecycle(tmp_path):
    db_path = tmp_path / "tables_test.db"
    Session = init_db(f"sqlite:///{db_path}")
    session = Session()
    manager = TableManager(session)

    # Open new table
    t1 = manager.open_table(1)
    assert t1.status == "open"

    # Occupy table
    t1 = manager.occupy_table(1)
    assert t1.status == "occupied"

    # Close table
    t1 = manager.close_table(1)
    assert t1.status == "closed"


def test_merge_tables(tmp_path):
    db_path = tmp_path / "merge_test.db"
    Session = init_db(f"sqlite:///{db_path}")
    session = Session()
    manager = TableManager(session)

    # Create 2 tables
    t1 = manager.open_table(1)
    t2 = manager.open_table(2)

    # Add order to table 2
    order = OrderDB(table=t2)
    session.add(order)
    session.commit()
    assert len(t2.orders) == 1

    # Merge table 2 into table 1
    t1 = manager.merge_tables(1, 2)
    assert len(t1.orders) == 1
    assert t2.status == "closed"



from mm_pos.db import init_db, MenuItemDB, OrderDB, OrderItemDB, PaymentDB, UserDB


def test_create_order_with_items_and_payment(tmp_path):
    db_path = tmp_path / "test.db"
    Session = init_db(f"sqlite:///{db_path}")
    session = Session()

    # Add menu item
    pizza = MenuItemDB(name="Pizza", price=12.5, category="Food")
    session.add(pizza)
    session.commit()

    # Create order
    order = OrderDB(table_number=1)
    session.add(order)
    session.commit()

    # Add order item
    session.add(OrderItemDB(order_id=order.id, menu_item_id=pizza.id, qty=2))
    session.commit()

    # Add payment
    payment = PaymentDB(order_id=order.id, method="cash", amount_given=30.00)
    session.add(payment)
    session.commit()

    # Assertions
    saved_order = session.query(OrderDB).first()
    assert saved_order.table_number == 1
    assert saved_order.items[0].qty == 2
    assert saved_order.payments[0].method == "cash"


def test_create_user_and_link_to_order_and_payment(tmp_path):
    db_path = tmp_path / "user_test.db"
    Session = init_db(f"sqlite:///{db_path}")
    session = Session()

    # Create user with secure pin
    alice = UserDB(name="Alice", role="waiter")
    alice.set_pin("1234")  # ✅ hash pin before saving
    session.add(alice)
    session.commit()

    # Assign order to user
    order = OrderDB(table_number=5, user=alice)
    session.add(order)
    session.commit()

    # Assign payment to same user
    payment = PaymentDB(order=order, method="cash", amount_given=50.0, user=alice)
    session.add(payment)
    session.commit()

    # Assertions
    saved_user = session.query(UserDB).first()
    assert saved_user.name == "Alice"
    assert saved_user.orders[0].table_number == 5
    assert saved_user.payments[0].method == "cash"



import pytest
from mm_pos.db import (
    init_db,
    MenuItemDB,
    MenuItemInventoryDB,
    OrderDB,
    OrderItemDB,
)
from mm_pos.inventory import InventoryManager


def test_inventory_deduction(tmp_path):
    db_path = tmp_path / "inv_test.db"
    Session = init_db(f"sqlite:///{db_path}")
    session = Session()
    inv = InventoryManager(session)

    patties = inv.add_stock("Burger Patty", 10)

    burger = MenuItemDB(name="Burger", price=9.99)
    session.add(burger)
    session.commit()

    link = MenuItemInventoryDB(
        menu_item_id=burger.id, inventory_id=patties.id, amount_used=1
    )
    session.add(link)
    session.commit()

    order = OrderDB(table_number=1)
    session.add(order)
    session.commit()

    oi = OrderItemDB(order_id=order.id, menu_item_id=burger.id, qty=2)
    session.add(oi)
    session.commit()

    inv.deduct_for_order(order.items)

    assert inv.get_stock("Burger Patty") == 8


def test_insufficient_stock_raises(tmp_path):
    db_path = tmp_path / "inv_fail.db"
    Session = init_db(f"sqlite:///{db_path}")
    session = Session()
    inv = InventoryManager(session)

    patties = inv.add_stock("Burger Patty", 1)

    burger = MenuItemDB(name="Burger", price=9.99)
    session.add(burger)
    session.commit()

    link = MenuItemInventoryDB(
        menu_item_id=burger.id, inventory_id=patties.id, amount_used=2
    )
    session.add(link)
    session.commit()

    order = OrderDB(table_number=1)
    session.add(order)
    session.commit()

    oi = OrderItemDB(order_id=order.id, menu_item_id=burger.id, qty=1)
    session.add(oi)
    session.commit()

    with pytest.raises(ValueError):
        inv.deduct_for_order(order.items)


def test_insufficient_stock_no_error_if_enforce_false(tmp_path):
    db_path = tmp_path / "inv_flexible.db"
    Session = init_db(f"sqlite:///{db_path}")
    session = Session()
    inv = InventoryManager(session)

    patties = inv.add_stock("Burger Patty", 1)

    burger = MenuItemDB(name="Burger", price=9.99)
    session.add(burger)
    session.commit()

    link = MenuItemInventoryDB(
        menu_item_id=burger.id, inventory_id=patties.id, amount_used=2
    )
    session.add(link)
    session.commit()

    order = OrderDB(table_number=1)
    session.add(order)
    session.commit()

    oi = OrderItemDB(order_id=order.id, menu_item_id=burger.id, qty=1)
    session.add(oi)
    session.commit()

    # With enforce=False, should NOT raise
    inv.deduct_for_order(order.items, enforce=False)

    # Stock should now be negative
    assert inv.get_stock("Burger Patty") == -1


def test_global_enforce_true_raises(tmp_path):
    db_path = tmp_path / "inv_enforce.db"
    Session = init_db(f"sqlite:///{db_path}")
    session = Session()
    inv = InventoryManager(session, enforce=True)  # strict mode

    patties = inv.add_stock("Burger Patty", 1)

    burger = MenuItemDB(name="Burger", price=9.99)
    session.add(burger)
    session.commit()

    link = MenuItemInventoryDB(
        menu_item_id=burger.id, inventory_id=patties.id, amount_used=2
    )
    session.add(link)
    session.commit()

    order = OrderDB(table_number=1)
    session.add(order)
    session.commit()

    oi = OrderItemDB(order_id=order.id, menu_item_id=burger.id, qty=1)
    session.add(oi)
    session.commit()

    with pytest.raises(ValueError):
        inv.deduct_for_order(order.items)


def test_global_enforce_false_allows_negative_stock(tmp_path):
    db_path = tmp_path / "inv_flex.db"
    Session = init_db(f"sqlite:///{db_path}")
    session = Session()
    inv = InventoryManager(session, enforce=False)  # flexible mode

    patties = inv.add_stock("Burger Patty", 1)

    burger = MenuItemDB(name="Burger", price=9.99)
    session.add(burger)
    session.commit()

    link = MenuItemInventoryDB(
        menu_item_id=burger.id, inventory_id=patties.id, amount_used=2
    )
    session.add(link)
    session.commit()

    order = OrderDB(table_number=1)
    session.add(order)
    session.commit()

    oi = OrderItemDB(order_id=order.id, menu_item_id=burger.id, qty=1)
    session.add(oi)
    session.commit()

    # No error in flexible mode
    inv.deduct_for_order(order.items)
    assert inv.get_stock("Burger Patty") == -1
